package checkers.quals;

import static java.lang.annotation.ElementType.*;

import java.lang.annotation.*;

/**
 * Specifies the annotations to be included in a type without having to provide
 * them explicitly.
 *
 * This annotations permits specifying multiple default qualifiers for
 * more than one type system.
 *
 * Example:
 * <code><pre>
 * \@DefaultQualifiers({
 *   \@DefaultQualifier("NonNull"),
 *   \@DefaultQualifier(value = "Interned", locations = ALL_EXCEPT_LOCALS),
 *   \@DefaultQualifier("Tainted")})
 * </pre></code>
 *
 * @see DefaultQualifier
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({CONSTRUCTOR, METHOD, FIELD, LOCAL_VARIABLE, PARAMETER, TYPE})
public @interface DefaultQualifiers {
    /** The default qualifier settings */
    DefaultQualifier[] value() default { };
}
